# -*- coding: utf-8 -*-
"""
Created on Mon Sep  2 15:49:46 2019

@author: msardar2
"""
from stats_arrays import *
import numpy as np
import pandas as pd
from pathlib import Path

class MC():
    """
    This class generates random number for Monte-Carlo simulations. This class is the interface to `stats_arrays <https://pypi.org/project/stats-arrays/>`_ package.
    
    The example below is showing the usage of ``stats_arrays``.
    
    :Example:
        
    >>> from stats_arrays import *
    >>> my_variables = UncertaintyBase.from_dicts(
    ...     {'loc': 2, 'scale': 0.5, 'uncertainty_type': NormalUncertainty.id},
    ...     {'loc': 1.5, 'minimum': 0, 'maximum': 10, 'uncertainty_type': TriangularUncertainty.id}
    ... )
    >>> my_variables
    array([(2.0, 0.5, nan, nan, nan, False, 3),
           (1.5, nan, nan, 0.0, 10.0, False, 5)],
        dtype=[('loc', '<f8'), ('scale', '<f8'), ('shape', '<f8'),
               ('minimum', '<f8'), ('maximum', '<f8'), ('negative', '?'),
               ('uncertainty_type', 'u1')])
    >>> my_rng = MCRandomNumberGenerator(my_variables)
    >>> my_rng.next()
    array([ 2.74414022,  3.54748507])
    
    .. _Class_MC: 
    
    
    """
    def __init__(self,input_dict):
        """ Initialize ``MC`` class
        
        :param input_dict: list of dictionaries that include input data (see the example)
        :type input_dict: list
        
        >>> from swolfpy_inputdata import MC
        >>> input_dict={'Cat1': {'Par1': {'Name': 'Name1','amount': 1.0,'unit': 'Unit1',
        ...                                 'uncertainty_type': 3,'loc': 1,'scale':0.2 ,'shape': None,
        ...                                 'minimum': None,'maximum': None,
        ...                                 'Reference': None,'Comment': None},
        ...                     'Par2': {'Name': 'Name2','amount': 1.5,'unit': 'Unit2',
        ...                                 'uncertainty_type': 3,'loc': 1.5,'scale': 0.4,'shape': None,
        ...                                 'minimum': None,'maximum': None,
        ...                                 'Reference': None,'Comment': None}}}
        >>> test_MC = MC(input_dict)
        >>> test_MC.setup_MC()
        >>> test_MC.gen_MC()
        [(('Cat1', 'Par1'), 1.0554408376879747),
         (('Cat1', 'Par2'), 1.9366617123732333)]
        """
        self.input_dict = input_dict
        
    def setup_MC(self,seed=None):
        """ Creates `MCRandomNumberGenerator` and store it in ``MC.rand`` attribute
    
        :param seed: seed for random number generation
        :type seed: int, optional
        """
        self.list_var = list()
        self.sens_analysis={}
        self.run_index=0
        self.max_run = 10000000
        k=0
        for x in self.input_dict.values():
            for y in x:
                self.list_var.append(x[y])
                if 'list' in x[y].keys():
                    self.sens_analysis[k] = x[y]['list']
                    self.max_run = min(self.max_run,len(x[y]['list']))
                k+=1
        self.Vars  = UncertaintyBase.from_dicts(*self.list_var)
        self.rand = MCRandomNumberGenerator(self.Vars,seed=seed)
      
    def gen_MC(self):
        """Generate random numbers and update data. It also returns a list of tuples include the name of parameter and generted number.
        
        :return: List of tuples include the name of parameter and generted number: ``[((parameter_category,parameter),generated_number)]``
        :rtype: list
        """
        data = self.rand.next()
        if len(self.sens_analysis) > 0:
            for x in self.sens_analysis.keys():
                data[x] = self.sens_analysis[x][self.run_index] 
            self.run_index+=1
            if self.run_index > self.max_run:
                raise ValueError('Number of runs are more than the number of defined inputs') 
        i=0
        variables = []
        for x in self.input_dict.keys():
            for y in self.input_dict[x]:
                if not np.isnan(data[i]):  
                    self.input_dict[x][y]['amount'] = data[i]
                    variables.append( ( (x , y) , data[i]) )
                i+=1
        return(variables)
		
    def create_uncertainty_from_inputs(self,sheet_name,process_data,seed=None):
        """Creates `MCRandomNumberGenerator` based on the data related to materials. 
        """
        self.process_data=process_data
        self.process_data_1=pd.read_excel(Path(__file__).parent/'Data/Material properties - process modles.xlsx', sheet_name = sheet_name, index_col = 'Parameter')
        self.uncertain_dict = dict()
        cols = list(self.process_data_1)
        for col in range(0,len(cols),7):
            self.uncertain_dict[cols[col]] = list()
            for val in range(len(self.process_data[cols[col]][3:])):
                self.uncertain_dict[cols[col]].append(dict())
                if not np.isnan(self.process_data_1[cols[col+1]][3+val]):
                    self.uncertain_dict[cols[col]][val]['uncertainty_type'] = int(self.process_data_1[cols[col+1]][3+val])
                    self.uncertain_dict[cols[col]][val]['loc'] = self.process_data_1[cols[col+2]][3+val]
                    self.uncertain_dict[cols[col]][val]['scale'] = self.process_data_1[cols[col+3]][3+val]
                    self.uncertain_dict[cols[col]][val]['shape'] = self.process_data_1[cols[col+4]][3+val]
                    self.uncertain_dict[cols[col]][val]['minimum'] = self.process_data_1[cols[col+5]][3+val]
                    self.uncertain_dict[cols[col]][val]['maximum'] = self.process_data_1[cols[col+6]][3+val]
                else:
                    self.uncertain_dict[cols[col]][val]['uncertainty_type'] = 1
							
        self.variables = dict()
        self.rng = dict()
        for key in self.uncertain_dict.keys():
            self.variables[key] = UncertaintyBase.from_dicts(*self.uncertain_dict[key])
            self.rng[key] = MCRandomNumberGenerator(self.variables[key],seed=seed)
		
    def uncertainty_input_next(self):
        """Generate random numbers and update the `process_data`
        """
        data = dict()
        variables = list()
        for key in self.rng.keys():
            data[key] = self.rng[key].next()
            for val in range(len(self.process_data[key][3:])):
                if not np.isnan(data[key][val]):			
                    self.process_data.at[(self.process_data_1.index.values[3+val]),key] = data[key][val]
                    variables.append(((key,self.process_data_1.index.values[3+val]),data[key][val]))
        return variables
	



# -*- coding: utf-8 -*-
"""
Created on Fri May  8 00:12:49 2020

@author: msmsa
"""
from swolfpy_inputdata import *

def test_all_inputs():
    A = LF_Input()
    A.setup_MC()
    assert isinstance(A.gen_MC(),list)
    
    assert WTE_Input()
    
    assert Comp_Input()
    
    assert AD_Input()
    
    assert SS_MRF_Input()
    
    assert Reproc_Input()
    
    assert SF_Col_Input()
    
    assert Technosphere_Input()
    
    assert CommonData()



# -*- coding: utf-8 -*-
"""
Created on Fri Nov 22 11:16:05 2019

@author: msardar2
"""
from pathlib import Path
import pandas as pd

class Technosphere_Input():
    def __init__(self,LCI_path=None,LCI_Reference_path=None,Ecospold2_Path=None):
            
        #Checking the path for technosphere LCI data and writing the technophere database
        if LCI_path:
            self.technosphere_path = LCI_path
        else:
            self.technosphere_path = str(Path(__file__).parent)+'\Data\Technosphere_LCI.csv'
        
        #Checking the path for technosphere LCI references
        if LCI_Reference_path:
            self.LCI_Reference_path = LCI_Reference_path
        else:
            self.LCI_Reference_path = str(Path(__file__).parent)+'\Data\Technosphere_References.csv'
        
        #Checking the path for Ecospold2
        if Ecospold2_Path:
            self.Ecospold2_Path = Ecospold2_Path
        else:
            self.Ecospold2_Path = str(Path(__file__).parent)+'\Data\Ecospold2'
        
        #Read the data
        self.LCI_swolfpy_data = pd.read_csv(self.technosphere_path)
        self.LCI_reference = pd.read_csv(self.LCI_Reference_path,index_col='swolfpy_technosphere_name')
        
        if self.LCI_reference['Reference_activity_id'].count()>0 and self.Ecospold2_Path==None:
            raise ValueError('User should select the path to ecospold files, because of keys in Technosphere_References.csv')


# -*- coding: utf-8 -*-
"""
Created on Tue Apr 21 20:45:14 2020

@author: msmsa
"""
import pandas as pd
import numpy as np
from .MC import *
from pathlib import Path
import ast

class InputData(MC):
    """
    ``InputData`` class reads the input data from the csv file and load them as class attributes. This class is inherited from the ``MC`` class. 
    
    Main functionalities include:
    loading data, updating data and generating random number for data based on the defined probabiliy distributions.
    """
    def __init__(self,input_data_path,eval_parameter=False):
        """Initialize ``InputData`` class
        
        :param input_data_path: absolute path to the input data file
        :type input_data_path: str
        :param eval_parameter: If the parameters are tuple instead of str, it will evalute their real value.
        :type eval_parameter: bool, optional
        """
        self.input_data_path = input_data_path            
        self.Data=pd.read_csv(self.input_data_path,dtype={'amount':float,'uncertainty_type':float,'loc':float,
                                                      'scale':float,'shape':float,'minimum':float,'maximum':float})

        if eval_parameter:     
            self.Data['Parameter']=self.Data['Parameter'].apply(ast.literal_eval)
        
        # Setting uncertainty type to 0 : Undefined ; when it is not defined
        self.Data['uncertainty_type'].fillna(0,inplace=True)
        #self.Data=self.Data.where((pd.notnull(self.Data)),None) 
        self.Input_dict = {}
        self.keys = self.Data.columns[3:]
        for i in range(len(self.Data)):
            if self.Data.Category[i] not in self.Input_dict.keys():
                exec("self.%s = {}" % self.Data.Dictonary_Name[i])
                exec("self.Input_dict[self.Data.Category[i]] = self.%s" % self.Data.Dictonary_Name[i])
                exec("self.%s[self.Data.Parameter[i]] = dict(zip(self.keys,self.Data.loc[i,'Name':]))" % self.Data.Dictonary_Name[i])
            else:
                exec("self.%s[self.Data.Parameter[i]] = dict(zip(self.keys,self.Data.loc[i,'Name':]))" % self.Data.Dictonary_Name[i])

### Update_Input
    def Update_input(self,NewData):
        """ Get a new DataFrame and update the ``data`` in ``InputData`` class.
        
        :param NewData:
        :type NewData: 'pandas.DataFrame'
        """
        for i in NewData.index:
            exec("self.%s[NewData.Parameter[i]] = dict(zip(self.keys,NewData.loc[i,'Name':]))" %NewData.Dictonary_Name[i])
            self.Data.loc[i]=NewData.loc[i]
            
### Monte_carlo          
    def setup_MC(self,seed=None):
        """ Initialize the parent class (``MC``) and create ``MCRandomNumberGenerator`` based on the data for uncertainty distributions via
        calling ``MC.setupMC()`` method.
        
        :param seed: seed for random number generation
        :type seed: int, optional
        
        .. seealso:: Class_MC_
        """
        super().__init__(self.Input_dict)
        super().setup_MC(seed)


#!/usr/bin/env python

"""Tests for `swolfpy_inputdata` package."""

import pytest     
from swolfpy_inputdata import InputData
from copy import deepcopy
from pathlib import Path

@pytest.fixture(scope="module")
def Input_Data():
    A = InputData(Path(__file__).parent/'Test_input.csv')
    yield A


def test_InputData(Input_Data):
    assert list(Input_Data.Data.columns) == ['Category', 'Dictonary_Name', 'Parameter', 'Name', 'amount', 'unit',
       'uncertainty_type', 'loc', 'scale', 'shape', 'minimum', 'maximum',
       'Reference', 'Comment']
    assert Input_Data.Dic1
    assert Input_Data.Dic2
    assert Input_Data.Dic1['Par1']['Name']=='Name1'
    assert Input_Data.Dic1['Par1']['amount']==1
    assert Input_Data.Dic1['Par1']['uncertainty_type']==0
    
def test_update_InputData(Input_Data):
    newdata = deepcopy(Input_Data.Data)
    #add uncertainty to Par2 of Dic1
    newdata.loc[1,'amount'] = 3
    newdata.loc[1,'uncertainty_type'] = 3
    newdata.loc[1,'loc'] = 3
    newdata.loc[1,'scale'] = 0.5
    
    Input_Data.Update_input(newdata)
    assert Input_Data.Dic1['Par2']['uncertainty_type']==3
    assert Input_Data.Dic1['Par2']['amount']==3
    assert Input_Data.Dic1['Par2']['loc']==3
    assert Input_Data.Dic1['Par2']['scale']==0.5
    

def test_setup_MC(Input_Data):    
    Input_Data.setup_MC()
    assert Input_Data.Dic1['Par2']['amount']==3
    assert isinstance(Input_Data.gen_MC(),list)
    assert isinstance(Input_Data.gen_MC()[0],tuple)
    assert Input_Data.gen_MC()[0][0] == ('Cat1','Par2')
    assert abs(Input_Data.gen_MC()[0][1]-3)<=2 
    

    
    
    
    



# -*- coding: utf-8 -*-
"""
Created on Mon Jul  1 20:05:32 2019

@author: msardar2
"""
from .InputData import InputData
from pathlib import Path
import pandas as pd

class CommonData(InputData):
    def __init__(self,input_data_path = None):
        if input_data_path:
            self.input_data_path = input_data_path
        else:
            self.input_data_path = Path(__file__).parent/'Data/CommonData.csv'
        
        # Initialize the superclass 
        super().__init__(self.input_data_path)

        ### Read Material properties
        self.Material_Properties=pd.read_excel(Path(__file__).parent/"Data/Material properties.xlsx",index_col = 'Materials')
        self.Material_Properties.fillna(0,inplace=True)

### Materials
        self.Index = ['Yard_Trimmings_Leaves', 'Yard_Trimmings_Grass', 'Yard_Trimmings_Branches', 'Food_Waste_Vegetable', 'Food_Waste_Non_Vegetable',
         'Wood', 'Wood_Other', 'Textiles', 'Rubber_Leather', 'Newsprint', 'Corr_Cardboard', 'Office_Paper', 'Magazines', 'third_Class_Mail',
         'Folding_Containers', 'Paper_Bags', 'Mixed_Paper', 'Paper_Non_recyclable', 'HDPE_Translucent_Containers', 'HDPE_Pigmented_Containers',
         'PET_Containers', 'Plastic_Other_1_Polypropylene', 'Plastic_Other_2', 'Mixed_Plastic', 'Plastic_Film', 'Plastic_Non_Recyclable',
         'Ferrous_Cans', 'Ferrous_Metal_Other', 'Aluminum_Cans', 'Aluminum_Foil', 'Aluminum_Other', 'Ferrous_Non_recyclable', 'Al_Non_recyclable',
         'Glass_Brown', 'Glass_Green', 'Glass_Clear', 'Mixed_Glass', 'Glass_Non_recyclable', 'Misc_Organic', 'Misc_Inorganic', 'E_waste',
         'Aerobic_Residual', 'Anaerobic_Residual', 'Bottom_Ash', 'Fly_Ash', 'Diapers_and_sanitary_products', 'Waste_Fraction_47', 'Waste_Fraction_48',
         'Waste_Fraction_49', 'Waste_Fraction_50', 'Waste_Fraction_51', 'Waste_Fraction_52', 'Waste_Fraction_53', 'Waste_Fraction_54', 'Waste_Fraction_55',
         'Waste_Fraction_56', 'Waste_Fraction_57', 'Waste_Fraction_58', 'Waste_Fraction_59', 'Waste_Fraction_60']

# Recycling products index
        self.Reprocessing_Index = ['Al','Fe','Cu',
                                   'OCC','Mixed_Paper','ONP','OFF','Fiber_Other',
                                   'Brown_glass','Clear_glass','Green_glass','Mixed_Glass',
                                   'PET','HDPE_Unsorted','HDPE_P','HDPE_T','PVC','LDPE_Film','Polypropylene','Polystyrene','Plastic_Other','Mixed_Plastic']

# Collection products index
        self.Collection_Index = ['RWC','SSR','DSR','MSR','LV','SSYW','SSO','DryRes','REC','WetRes','MRDO','SSYWDO','MSRDO']        

# Waste products index
        self.Waste_Pr_Index = ['Bottom_Ash','Fly_Ash','Separated_Organics','Other_Residual','RDF']

# all waste_pr_index
        self.All_Waste_Pr_Index =  self.Waste_Pr_Index + self.Collection_Index + self.Reprocessing_Index
        






















